/* network.h -- network headers */

#ifndef NETWORK_H
#define NETWORK_H

/* includes */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

/* net specific includes */
#include <netinet/in.h>
#include <arpa/inet.h>

/* defines */
#define MAX_CONNECTIONS 32

#define IP_DIGITS 16
#define C_IN_USE (1<<0)
#define C_INFO_A (1<<1)
#define C_SOCKET (1<<2)
#define C_ESTABL (1<<3)
#define C_HANGUP (1<<4)

#define SEND_N_MAX 128

#define N_SUCCESS 1
#define N_ERROR -1
#define N_E_IN_USE -2
#define N_E_NO_INFO -3
#define N_E_CLOSE -4
#define N_E_CONNECT -5
#define N_E_NC -6
#define N_E_ACCEPT -7
#define N_E_MAXC -8

#define MAX_LISTEN_QUEUE 32

/* net specific variables */
typedef struct s_connection {
  int fd;
  char ip[IP_DIGITS];
  in_port_t port;
  unsigned char status;
  unsigned short cap; /* capabilities */
} t_connection;

typedef struct s_net {
  int l_fd; /* listen file descriptor */
  in_port_t l_port;
  unsigned short cap;
  /* limited connections by now -- replaced by list management later */
  int c_count;
  t_connection connection[MAX_CONNECTIONS];
  unsigned int sendmask; /* 32 bits for maximum of 32 connections */
} t_net;

/* function prototypes */
int network_init(t_net *net);
int network_shutdown(t_net *net);
int network_set_listen_port(t_net *net,in_port_t port);
int network_manage_connection(t_net *net);
int network_connect(t_net *net,int channel);
int network_close(t_net *net,int channel);
int network_close_all(t_net *net);
int network_set_connection_info(t_net *net,int channel,char *ip,int port);
int network_select(t_net *net,int channel);
int network_deselect(t_net *net,int channel);
int network_manage_incoming(t_net *net);
int network_send(int fd,unsigned char *data,int datasize);
int network_receive(int fd,unsigned char *data,int datasize);

#endif
